@@ -69,6 +69,8 @@ enum dr_dump_rec_type {
 	DR_DUMP_REC_TYPE_ACTION_VPORT = 3408,
 	DR_DUMP_REC_TYPE_ACTION_DECAP_L2 = 3409,
 	DR_DUMP_REC_TYPE_ACTION_DECAP_L3 = 3410,
+	DR_DUMP_REC_TYPE_ACTION_DEVX_TIR = 3411,
+	DR_DUMP_REC_TYPE_ACTION_METER = 3414,
 };
 
 static uint64_t dr_dump_icm_to_idx(uint64_t icm_addr)
@@ -88,7 +90,7 @@ static int dr_dump_rule_action_mem(FILE *f, const uint64_t rule_id,
 				   struct dr_rule_action_member *action_mem)
 {
 	struct mlx5dv_dr_action *action = action_mem->action;
-	const uint64_t action_id = (uint64_t)action;
+	const uint64_t action_id = (uint64_t) (uintptr_t) action;
 	int ret;
 
 	switch (action->action_type) {
@@ -97,14 +99,20 @@ static int dr_dump_rule_action_mem(FILE *f, const uint64_t rule_id,
 			      DR_DUMP_REC_TYPE_ACTION_DROP, action_id, rule_id);
 		break;
 	case DR_ACTION_TYP_FT:
-		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
+		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x,0x%" PRIx64 "\n",
 			      DR_DUMP_REC_TYPE_ACTION_FT, action_id, rule_id,
-			      action->dest_tbl->devx_obj->object_id);
+			      action->dest_tbl->devx_obj->object_id,
+			      (uint64_t)(uintptr_t)action->dest_tbl);
 		break;
 	case DR_ACTION_TYP_QP:
-		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
-			      DR_DUMP_REC_TYPE_ACTION_QP, action_id, rule_id,
-			      action->qp->qp_num);
+		if (action->dest_qp.is_qp)
+			ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
+				      DR_DUMP_REC_TYPE_ACTION_QP, action_id,
+				      rule_id, action->dest_qp.qp->qp_num);
+		else
+			ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%" PRIx64 "\n",
+				      DR_DUMP_REC_TYPE_ACTION_DEVX_TIR, action_id,
+				      rule_id, action->dest_qp.devx_tir->rx_icm_addr);
 		break;
 	case DR_ACTION_TYP_CTR:
 		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
@@ -113,12 +121,12 @@ static int dr_dump_rule_action_mem(FILE *f, const uint64_t rule_id,
 			      action->ctr.offset);
 		break;
 	case DR_ACTION_TYP_TAG:
-		ret = fprintf(f, "%d,,0x%" PRIx64 ",0x%" PRIx64 "0x%x\n",
+		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
 			      DR_DUMP_REC_TYPE_ACTION_TAG, action_id, rule_id,
 			      action->flow_tag);
 		break;
 	case DR_ACTION_TYP_MODIFY_HDR:
-		ret = fprintf(f, "%d,,0x%" PRIx64 ",0x%" PRIx64 "0x%x\n",
+		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x\n",
 			      DR_DUMP_REC_TYPE_ACTION_MODIFY_HDR, action_id,
 			      rule_id, action->rewrite.index);
 		break;
@@ -147,6 +155,16 @@ static int dr_dump_rule_action_mem(FILE *f, const uint64_t rule_id,
 			      DR_DUMP_REC_TYPE_ACTION_ENCAP_L3, action_id,
 			      rule_id, action->reformat.dvo->object_id);
 		break;
+	case DR_ACTION_TYP_METER:
+		ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%" PRIx64 ",0x%x,0x%" PRIx64 ",0x%" PRIx64 "\n",
+			      DR_DUMP_REC_TYPE_ACTION_METER,
+			      action_id,
+			      rule_id,
+			      (uint64_t)(uintptr_t)action->meter.next_ft,
+			      action->meter.devx_obj->object_id,
+			      action->meter.rx_icm_addr,
+			      action->meter.tx_icm_addr);
+		break;
 	default:
 		return 0;
 	}
@@ -196,7 +214,7 @@ static int dr_dump_rule_rx_tx(FILE *f, struct dr_rule_rx_tx *rule_rx_tx,
 static int dr_dump_rule(FILE *f, struct mlx5dv_dr_rule *rule)
 {
 	struct dr_rule_action_member *action_mem;
-	const uint64_t rule_id = (uint64_t)rule;
+	const uint64_t rule_id = (uint64_t) (uintptr_t) rule;
 	struct dr_rule_rx_tx *rx = &rule->rx;
 	struct dr_rule_rx_tx *tx = &rule->tx;
 	int ret;
@@ -204,7 +222,7 @@ static int dr_dump_rule(FILE *f, struct mlx5dv_dr_rule *rule)
 	ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 "\n",
 		      DR_DUMP_REC_TYPE_RULE,
 		      rule_id,
-		      (uint64_t)rule->matcher);
+		      (uint64_t) (uintptr_t) rule->matcher);
 	if (ret < 0)
 		return ret;
 
@@ -341,7 +359,7 @@ static int dr_dump_matcher_rx_tx(FILE *f, bool is_rx,
 
 	ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",%d,0x%" PRIx64 ",0x%" PRIx64 "\n",
 		      rec_type,
-		      (uint64_t)matcher_rx_tx,
+		      (uint64_t) (uintptr_t) matcher_rx_tx,
 		      matcher_id,
 		      matcher_rx_tx->num_of_builders,
 		      dr_dump_icm_to_idx(matcher_rx_tx->s_htbl->chunk->icm_addr),
@@ -366,12 +384,12 @@ static int dr_dump_matcher(FILE *f, struct mlx5dv_dr_matcher *matcher)
 	uint64_t matcher_id;
 	int ret;
 
-	matcher_id = (uint64_t)matcher;
+	matcher_id = (uint64_t) (uintptr_t) matcher;
 
 	ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",%d\n",
 		      DR_DUMP_REC_TYPE_MATCHER,
 		      matcher_id,
-		      (uint64_t)matcher->tbl,
+		      (uint64_t) (uintptr_t) matcher->tbl,
 		      matcher->prio);
 	if (ret < 0)
 		return ret;
@@ -464,7 +482,7 @@ static int dr_dump_table(FILE *f, struct mlx5dv_dr_table *table)
 
 	ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",%d,%d\n",
 		      DR_DUMP_REC_TYPE_TABLE,
-		      (uint64_t) table,
+		      (uint64_t) (uintptr_t) table,
 		      dr_domain_id_calc(table->dmn->type),
 		      table->table_type,
 		      table->level);
@@ -473,13 +491,13 @@ static int dr_dump_table(FILE *f, struct mlx5dv_dr_table *table)
 
 	if (!dr_is_root_table(table)) {
 		if (rx->nic_dmn) {
-			ret = dr_dump_table_rx_tx(f, true, rx, (uint64_t)table);
+			ret = dr_dump_table_rx_tx(f, true, rx, (uint64_t) (uintptr_t) table);
 			if (ret < 0)
 				return ret;
 		}
 
 		if (tx->nic_dmn) {
-			ret = dr_dump_table_rx_tx(f, false, tx, (uint64_t)table);
+			ret = dr_dump_table_rx_tx(f, false, tx, (uint64_t) (uintptr_t) table);
 			if (ret < 0)
 				return ret;
 		}
@@ -529,7 +547,7 @@ static int dr_dump_send_ring(FILE *f, struct dr_send_ring *ring,
 
 	ret = fprintf(f, "%d,0x%" PRIx64 ",0x%" PRIx64 ",0x%x,0x%x\n",
 		      DR_DUMP_REC_TYPE_DOMAIN_SEND_RING,
-		      (uint64_t)ring,
+		      (uint64_t) (uintptr_t) ring,
 		      domain_id,
 		      ring->cq.cqn,
 		      ring->qp->obj->object_id);
