@@ -120,7 +120,7 @@ static bool dr_mask_is_ttl_set(struct dr_match_spec *spec)
 	(_misc2)._inner_outer##_first_mpls_s_bos || \
 	(_misc2)._inner_outer##_first_mpls_ttl)
 
-static bool dr_mask_is_gre_set(struct dr_match_misc *misc)
+static bool dr_mask_is_tnl_gre_set(struct dr_match_misc *misc)
 {
 	return (misc->gre_key_h || misc->gre_key_l ||
 		misc->gre_protocol || misc->gre_c_present ||
@@ -133,12 +133,12 @@ static bool dr_mask_is_gre_set(struct dr_match_misc *misc)
 	(_misc2).outer_first_mpls_over_##gre_udp##_s_bos || \
 	(_misc2).outer_first_mpls_over_##gre_udp##_ttl)
 
-#define DR_MASK_IS_FLEX_PARSER_0_SET(_misc2) ( \
+#define DR_MASK_IS_TNL_MPLS_SET(_misc2) ( \
 	DR_MASK_IS_OUTER_MPLS_OVER_GRE_UDP_SET(_misc2, gre) || \
 	DR_MASK_IS_OUTER_MPLS_OVER_GRE_UDP_SET(_misc2, udp))
 
 static bool
-dr_mask_is_misc3_vxlan_gpe_set(struct dr_match_misc3 *misc3)
+dr_mask_is_vxlan_gpe_set(struct dr_match_misc3 *misc3)
 {
 	return misc3->outer_vxlan_gpe_vni ||
 	       misc3->outer_vxlan_gpe_next_protocol ||
@@ -146,21 +146,21 @@ dr_mask_is_misc3_vxlan_gpe_set(struct dr_match_misc3 *misc3)
 }
 
 static bool
-dr_matcher_supp_flex_parser_vxlan_gpe(struct dr_devx_caps *caps)
+dr_matcher_supp_vxlan_gpe(struct dr_devx_caps *caps)
 {
 	return caps->flex_protocols &
 	       MLX5_FLEX_PARSER_VXLAN_GPE_ENABLED;
 }
 
 static bool
-dr_mask_is_flex_parser_tnl_vxlan_gpe_set(struct dr_match_param *mask,
-					 struct mlx5dv_dr_domain *dmn)
+dr_mask_is_tnl_vxlan_gpe(struct dr_match_param *mask,
+			 struct mlx5dv_dr_domain *dmn)
 {
-	return dr_mask_is_misc3_vxlan_gpe_set(&mask->misc3) &&
-	       dr_matcher_supp_flex_parser_vxlan_gpe(&dmn->info.caps);
+	return dr_mask_is_vxlan_gpe_set(&mask->misc3) &&
+	       dr_matcher_supp_vxlan_gpe(&dmn->info.caps);
 }
 
-static bool dr_mask_is_misc_geneve_set(struct dr_match_misc *misc)
+static bool dr_mask_is_tnl_geneve_set(struct dr_match_misc *misc)
 {
 	return misc->geneve_vni ||
 	       misc->geneve_oam ||
@@ -169,46 +169,64 @@ static bool dr_mask_is_misc_geneve_set(struct dr_match_misc *misc)
 }
 
 static bool
-dr_matcher_supp_flex_parser_geneve(struct dr_devx_caps *caps)
+dr_matcher_supp_tnl_geneve(struct dr_devx_caps *caps)
 {
 	return caps->flex_protocols &
 	       MLX5_FLEX_PARSER_GENEVE_ENABLED;
 }
 
 static bool
-dr_mask_is_flex_parser_tnl_geneve_set(struct dr_match_param *mask,
-				      struct mlx5dv_dr_domain *dmn)
+dr_mask_is_tnl_geneve(struct dr_match_param *mask,
+		      struct mlx5dv_dr_domain *dmn)
 {
-	return dr_mask_is_misc_geneve_set(&mask->misc) &&
-	       dr_matcher_supp_flex_parser_geneve(&dmn->info.caps);
+	return dr_mask_is_tnl_geneve_set(&mask->misc) &&
+	       dr_matcher_supp_tnl_geneve(&dmn->info.caps);
 }
 
-static bool dr_mask_is_misc3_gtpu_set(struct dr_match_misc3 *misc3)
+static bool dr_mask_is_tnl_gtpu_set(struct dr_match_misc3 *misc3)
 {
-	return misc3->gtpu_flags ||
-	       misc3->gtpu_msg_type ||
-	       misc3->gtpu_teid;
+	return misc3->gtpu_flags || misc3->gtpu_msg_type || misc3->gtpu_teid;
 }
 
-static bool dr_matcher_supp_flex_parser_gtpu(struct dr_devx_caps *caps)
+static bool dr_matcher_supp_tnl_gtpu(struct dr_devx_caps *caps)
 {
-	return caps->flex_protocols &
-	       MLX5_FLEX_PARSER_GTPU_ENABLED;
+	return caps->flex_protocols & MLX5_FLEX_PARSER_GTPU_ENABLED;
 }
 
-static bool dr_mask_is_flex_parser_tnl_gtpu_set(struct dr_match_param *mask,
-					 struct mlx5dv_dr_domain *dmn)
+static bool dr_mask_is_tnl_gtpu(struct dr_match_param *mask,
+				struct mlx5dv_dr_domain *dmn)
 {
-	return dr_mask_is_misc3_gtpu_set(&mask->misc3) &&
-	       dr_matcher_supp_flex_parser_gtpu(&dmn->info.caps);
+	return dr_mask_is_tnl_gtpu_set(&mask->misc3) &&
+	       dr_matcher_supp_tnl_gtpu(&dmn->info.caps);
 }
 
-static bool dr_mask_is_flex_parser_icmpv6_set(struct dr_match_misc3 *misc3)
+static inline int dr_matcher_supp_icmp_v4(struct dr_devx_caps *caps)
+{
+	return caps->flex_protocols & MLX5_FLEX_PARSER_ICMP_V4_ENABLED;
+}
+
+static inline int dr_matcher_supp_icmp_v6(struct dr_devx_caps *caps)
+{
+	return caps->flex_protocols & MLX5_FLEX_PARSER_ICMP_V6_ENABLED;
+}
+
+static bool dr_mask_is_icmpv6_set(struct dr_match_misc3 *misc3)
 {
 	return (misc3->icmpv6_type || misc3->icmpv6_code ||
 		misc3->icmpv6_header_data);
 }
 
+static bool dr_mask_is_icmp(struct dr_match_param *mask,
+			    struct mlx5dv_dr_domain *dmn)
+{
+	if (DR_MASK_IS_ICMPV4_SET(&mask->misc3))
+		return dr_matcher_supp_icmp_v4(&dmn->info.caps);
+	else if (dr_mask_is_icmpv6_set(&mask->misc3))
+		return dr_matcher_supp_icmp_v6(&dmn->info.caps);
+
+	return false;
+}
+
 static bool dr_mask_is_wqe_metadata_set(struct dr_match_misc2 *misc2)
 {
 	return misc2->metadata_reg_a;
@@ -238,7 +256,7 @@ static int dr_matcher_set_ste_builders(struct mlx5dv_dr_matcher *matcher,
 	struct dr_ste_build *sb = nic_matcher->ste_builder;
 	struct mlx5dv_dr_domain *dmn = matcher->tbl->dmn;
 	struct dr_match_param mask = {};
-	struct dr_match_misc3 *misc3;
+	bool allow_empty_match = false;
 	bool inner, rx;
 	uint8_t ipv;
 	int idx = 0;
@@ -267,6 +285,15 @@ static int dr_matcher_set_ste_builders(struct mlx5dv_dr_matcher *matcher,
 	if (ret)
 		return ret;
 
+	/* Optimize RX pipe by reducing source port match, since
+	 * the FDB RX part is conneted only to the wire.
+	 */
+	if (dmn->type == MLX5DV_DR_DOMAIN_TYPE_FDB &&
+	    rx && mask.misc.source_port) {
+		mask.misc.source_port = 0;
+		allow_empty_match = true;
+	}
+
 	/* Outer */
 	if (matcher->match_criteria & (DR_MATCHER_CRITERIA_OUTER |
 				       DR_MATCHER_CRITERIA_MISC |
@@ -286,21 +313,14 @@ static int dr_matcher_set_ste_builders(struct mlx5dv_dr_matcher *matcher,
 
 		if (dr_mask_is_gvmi_or_qpn_set(&mask.misc) &&
 		    (dmn->type == MLX5DV_DR_DOMAIN_TYPE_FDB ||
-		     dmn->type == MLX5DV_DR_DOMAIN_TYPE_NIC_RX)) {
-			ret = dr_ste_build_src_gvmi_qpn(&sb[idx++], &mask,
-							&dmn->info.caps,
-							inner, rx);
-			if (ret)
-				return ret;
-		}
+		     dmn->type == MLX5DV_DR_DOMAIN_TYPE_NIC_RX))
+			dr_ste_build_src_gvmi_qpn(&sb[idx++], &mask,
+						  &dmn->info.caps,
+						  inner, rx);
 
 		if (dr_mask_is_smac_set(&mask.outer) &&
-		    dr_mask_is_dmac_set(&mask.outer)) {
-			ret = dr_ste_build_eth_l2_src_des(&sb[idx++], &mask,
-							  inner, rx);
-			if (ret)
-				return ret;
-		}
+		    dr_mask_is_dmac_set(&mask.outer))
+			dr_ste_build_eth_l2_src_dst(&sb[idx++], &mask, inner, rx);
 
 		if (dr_mask_is_smac_set(&mask.outer))
 			dr_ste_build_eth_l2_src(&sb[idx++], &mask, inner, rx);
@@ -309,13 +329,13 @@ static int dr_matcher_set_ste_builders(struct mlx5dv_dr_matcher *matcher,
 			dr_ste_build_eth_l2_dst(&sb[idx++], &mask, inner, rx);
 
 		if (ipv == 4) {
-			if (dr_mask_is_ipv4_5_tuple_set(&mask.outer))
-				dr_ste_build_eth_l3_ipv4_5_tuple(&sb[idx++], &mask,
-								 inner, rx);
-
 			if (dr_mask_is_ttl_set(&mask.outer))
 				dr_ste_build_eth_l3_ipv4_misc(&sb[idx++], &mask,
 							      inner, rx);
+
+			if (dr_mask_is_ipv4_5_tuple_set(&mask.outer))
+				dr_ste_build_eth_l3_ipv4_5_tuple(&sb[idx++], &mask,
+								 inner, rx);
 		} else if (ipv == 6) {
 			if (dr_mask_is_dst_addr_set(&mask.outer))
 				dr_ste_build_eth_l3_ipv6_dst(&sb[idx++], &mask,
@@ -326,19 +346,19 @@ static int dr_matcher_set_ste_builders(struct mlx5dv_dr_matcher *matcher,
 							     inner, rx);
 
 			if (DR_MASK_IS_ETH_L4_SET(mask.outer, mask.misc, outer))
-				dr_ste_build_ipv6_l3_l4(&sb[idx++], &mask,
-							inner, rx);
+				dr_ste_build_eth_ipv6_l3_l4(&sb[idx++], &mask,
+							    inner, rx);
 		}
 
-		if (dr_mask_is_flex_parser_tnl_vxlan_gpe_set(&mask, dmn))
-			dr_ste_build_flex_parser_tnl_vxlan_gpe(&sb[idx++], &mask,
-							       inner, rx);
-		else if (dr_mask_is_flex_parser_tnl_geneve_set(&mask, dmn))
-			dr_ste_build_flex_parser_tnl_geneve(&sb[idx++], &mask,
-							    inner, rx);
-		else if (dr_mask_is_flex_parser_tnl_gtpu_set(&mask, dmn))
-			dr_ste_build_flex_parser_tnl_gtpu(&sb[idx++], &mask,
-							    inner, rx);
+		if (dr_mask_is_tnl_vxlan_gpe(&mask, dmn))
+			dr_ste_build_tnl_vxlan_gpe(&sb[idx++], &mask,
+						   inner, rx);
+		else if (dr_mask_is_tnl_geneve(&mask, dmn))
+			dr_ste_build_tnl_geneve(&sb[idx++], &mask,
+						inner, rx);
+		else if (dr_mask_is_tnl_gtpu(&mask, dmn))
+			dr_ste_build_tnl_gtpu(&sb[idx++], &mask,
+					      inner, rx);
 
 		if (DR_MASK_IS_ETH_L4_MISC_SET(mask.misc3, outer))
 			dr_ste_build_eth_l4_misc(&sb[idx++], &mask, inner, rx);
@@ -346,22 +366,18 @@ static int dr_matcher_set_ste_builders(struct mlx5dv_dr_matcher *matcher,
 		if (DR_MASK_IS_FIRST_MPLS_SET(mask.misc2, outer))
 			dr_ste_build_mpls(&sb[idx++], &mask, inner, rx);
 
-		if (DR_MASK_IS_FLEX_PARSER_0_SET(mask.misc2))
-			dr_ste_build_flex_parser_0(&sb[idx++], &mask, inner, rx);
+		if (DR_MASK_IS_TNL_MPLS_SET(mask.misc2))
+			dr_ste_build_tnl_mpls(&sb[idx++], &mask, inner, rx);
 
-		misc3 = &mask.misc3;
-		if ((DR_MASK_IS_FLEX_PARSER_ICMPV4_SET(misc3) &&
-		     dr_matcher_supp_flex_parser_icmp_v4(&dmn->info.caps)) ||
-		    (dr_mask_is_flex_parser_icmpv6_set(&mask.misc3) &&
-		     dr_matcher_supp_flex_parser_icmp_v6(&dmn->info.caps))) {
-			ret = dr_ste_build_flex_parser_1(&sb[idx++],
-							 &mask, &dmn->info.caps,
-							 inner, rx);
+		if (dr_mask_is_icmp(&mask, dmn)) {
+			ret = dr_ste_build_icmp(&sb[idx++],
+						&mask, &dmn->info.caps,
+						inner, rx);
 			if (ret)
 				return ret;
 		}
-		if (dr_mask_is_gre_set(&mask.misc))
-			dr_ste_build_gre(&sb[idx++], &mask, inner, rx);
+		if (dr_mask_is_tnl_gre_set(&mask.misc))
+			dr_ste_build_tnl_gre(&sb[idx++], &mask, inner, rx);
 	}
 
 	/* Inner */
@@ -376,12 +392,8 @@ static int dr_matcher_set_ste_builders(struct mlx5dv_dr_matcher *matcher,
 			dr_ste_build_eth_l2_tnl(&sb[idx++], &mask, inner, rx);
 
 		if (dr_mask_is_smac_set(&mask.inner) &&
-		    dr_mask_is_dmac_set(&mask.inner)) {
-			ret = dr_ste_build_eth_l2_src_des(&sb[idx++],
-							  &mask, inner, rx);
-			if (ret)
-				return ret;
-		}
+		    dr_mask_is_dmac_set(&mask.inner))
+			dr_ste_build_eth_l2_src_dst(&sb[idx++], &mask, inner, rx);
 
 		if (dr_mask_is_smac_set(&mask.inner))
 			dr_ste_build_eth_l2_src(&sb[idx++], &mask, inner, rx);
@@ -390,13 +402,13 @@ static int dr_matcher_set_ste_builders(struct mlx5dv_dr_matcher *matcher,
 			dr_ste_build_eth_l2_dst(&sb[idx++], &mask, inner, rx);
 
 		if (ipv == 4) {
-			if (dr_mask_is_ipv4_5_tuple_set(&mask.inner))
-				dr_ste_build_eth_l3_ipv4_5_tuple(&sb[idx++], &mask,
-								 inner, rx);
-
 			if (dr_mask_is_ttl_set(&mask.inner))
 				dr_ste_build_eth_l3_ipv4_misc(&sb[idx++], &mask,
 							      inner, rx);
+
+			if (dr_mask_is_ipv4_5_tuple_set(&mask.inner))
+				dr_ste_build_eth_l3_ipv4_5_tuple(&sb[idx++], &mask,
+								 inner, rx);
 		} else if (ipv == 6) {
 			if (dr_mask_is_dst_addr_set(&mask.inner))
 				dr_ste_build_eth_l3_ipv6_dst(&sb[idx++], &mask,
@@ -407,8 +419,8 @@ static int dr_matcher_set_ste_builders(struct mlx5dv_dr_matcher *matcher,
 							     inner, rx);
 
 			if (DR_MASK_IS_ETH_L4_SET(mask.inner, mask.misc, inner))
-				dr_ste_build_ipv6_l3_l4(&sb[idx++], &mask,
-							inner, rx);
+				dr_ste_build_eth_ipv6_l3_l4(&sb[idx++], &mask,
+							    inner, rx);
 		}
 
 		if (DR_MASK_IS_ETH_L4_MISC_SET(mask.misc3, inner))
@@ -417,11 +429,14 @@ static int dr_matcher_set_ste_builders(struct mlx5dv_dr_matcher *matcher,
 		if (DR_MASK_IS_FIRST_MPLS_SET(mask.misc2, inner))
 			dr_ste_build_mpls(&sb[idx++], &mask, inner, rx);
 
-		if (DR_MASK_IS_FLEX_PARSER_0_SET(mask.misc2))
-			dr_ste_build_flex_parser_0(&sb[idx++], &mask, inner, rx);
+		if (DR_MASK_IS_TNL_MPLS_SET(mask.misc2))
+			dr_ste_build_tnl_mpls(&sb[idx++], &mask, inner, rx);
+
 	}
+
 	/* Empty matcher, takes all */
-	if (matcher->match_criteria == DR_MATCHER_CRITERIA_EMPTY)
+	if ((!idx && allow_empty_match) ||
+	    matcher->match_criteria == DR_MATCHER_CRITERIA_EMPTY)
 		dr_ste_build_empty_always_hit(&sb[idx++], rx);
 
 	if (idx == 0) {
