@@ -721,7 +721,7 @@ int dr_send_postsend_htbl(struct mlx5dv_dr_domain *dmn, struct dr_ste_htbl *htbl
 
 		/* Copy all ste's on the data buffer, need to add the bit_mask */
 		for (j = 0; j < num_stes_per_iter; j++) {
-			if (dr_ste_is_not_valid_entry(htbl->ste_arr[ste_index + j].hw_ste)) {
+			if (dr_ste_is_not_used(&htbl->ste_arr[ste_index + j])) {
 				memcpy(data + (j * DR_STE_SIZE),
 				       formated_ste, DR_STE_SIZE);
 			} else {
@@ -807,7 +807,8 @@ int dr_send_postsend_action(struct mlx5dv_dr_domain *dmn,
 	int ret;
 
 	send_info.write.addr	= (uintptr_t) action->rewrite.data;
-	send_info.write.length	= action->rewrite.chunk->byte_size;
+	send_info.write.length	= action->rewrite.num_of_actions *
+				  DR_MODIFY_ACTION_SIZE;
 	send_info.write.lkey	= 0;
 	send_info.remote_addr	= action->rewrite.chunk->mr_addr;
 	send_info.rkey		= action->rewrite.chunk->rkey;
@@ -837,17 +838,22 @@ static int dr_prepare_qp_to_rts(struct mlx5dv_dr_domain *dmn)
 	}
 
 	/* RTR */
-	ret = dr_devx_query_gid(dmn->ctx, port, gid_index, &rtr_attr.dgid_attr);
-	if (ret) {
-		dr_dbg(dmn, "can't read sgid of index %d\n", gid_index);
-		return ret;
-	}
-
 	rtr_attr.mtu		= mtu;
 	rtr_attr.qp_num		= dr_qp->obj->object_id;
 	rtr_attr.min_rnr_timer	= 12;
 	rtr_attr.port_num	= port;
-	rtr_attr.sgid_index	= gid_index;
+
+	/* Enable force-loopback on the QP */
+	if (dmn->info.caps.roce_caps.fl_rc_qp_when_roce_enabled) {
+		rtr_attr.fl = true;
+	} else {
+		ret = dr_devx_query_gid(dmn->ctx, port, gid_index, &rtr_attr.dgid_attr);
+		if (ret) {
+			dr_dbg(dmn, "can't read sgid of index %d\n", gid_index);
+			return ret;
+		}
+		rtr_attr.sgid_index = gid_index;
+	}
 
 	ret = dr_devx_modify_qp_init2rtr(dmn->ctx, dr_qp->obj,  &rtr_attr);
 	if (ret) {
