@@ -189,6 +189,13 @@ static uint16_t dr_ste_conv_bit_to_byte_mask(uint8_t *bit_mask)
 	return byte_mask;
 }
 
+static uint8_t *dr_ste_get_tag(uint8_t *hw_ste_p)
+{
+	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
+
+	return hw_ste->tag;
+}
+
 void dr_ste_set_bit_mask(uint8_t *hw_ste_p, uint8_t *bit_mask)
 {
 	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
@@ -546,38 +553,6 @@ void dr_ste_always_miss_addr(struct dr_ste *ste, uint64_t miss_addr)
 	dr_ste_set_always_miss((struct dr_hw_ste_format *)ste->hw_ste);
 }
 
-/*
- * The assumption here is that we don't update the ste->hw_ste if it is not
- * used ste, so it will be all zero, checking the next_lu_type.
- */
-bool dr_ste_is_not_valid_entry(uint8_t *p_hw_ste)
-{
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)p_hw_ste;
-
-	if (DR_STE_GET(general, hw_ste, next_lu_type) ==
-	    DR_STE_LU_TYPE_NOP)
-		return true;
-
-	return false;
-}
-
-bool dr_ste_not_used_ste(struct dr_ste *ste)
-{
-	return !atomic_load(&ste->refcount);
-}
-
-static uint16_t get_bits_per_mask(uint16_t byte_mask)
-{
-	uint16_t bits = 0;
-
-	while (byte_mask) {
-		byte_mask = byte_mask & (byte_mask - 1);
-		bits++;
-	}
-
-	return bits;
-}
-
 /* Init one ste as a pattern for ste data array */
 void dr_ste_set_formated_ste(uint16_t gvmi,
 			     struct dr_domain_rx_tx *nic_dmn,
@@ -626,20 +601,12 @@ int dr_ste_create_next_htbl(struct mlx5dv_dr_matcher *matcher,
 	struct dr_ste_htbl *next_htbl;
 
 	if (!dr_ste_is_last_in_rule(nic_matcher, ste->ste_chain_location)) {
-		uint32_t bits_in_mask;
 		uint8_t next_lu_type;
 		uint16_t byte_mask;
 
 		next_lu_type = DR_STE_GET(general, hw_ste, next_lu_type);
 		byte_mask = DR_STE_GET(general, hw_ste, byte_mask);
 
-		/* Don't allocate table more than required,
-		 * the size of the table defined via the byte_mask, so no need
-		 * to allocate more than that.
-		 */
-		bits_in_mask = get_bits_per_mask(byte_mask) * CHAR_BIT;
-		log_table_size = min_t(uint32_t, log_table_size, bits_in_mask);
-
 		next_htbl = dr_ste_htbl_alloc(dmn->ste_icm_pool,
 					      log_table_size,
 					      next_lu_type,
@@ -677,7 +644,7 @@ static void dr_ste_set_ctrl(struct dr_ste_htbl *htbl)
 
 	htbl->ctrl.may_grow = true;
 
-	if (htbl->chunk_size == DR_CHUNK_SIZE_MAX - 1)
+	if (htbl->chunk_size == DR_CHUNK_SIZE_MAX - 1 || !htbl->byte_mask)
 		htbl->ctrl.may_grow = false;
 
 	/* Threshold is 50%, one is added to table of size 1 */
@@ -819,7 +786,7 @@ int dr_ste_build_ste_arr(struct mlx5dv_dr_matcher *matcher,
 
 		dr_ste_set_bit_mask(ste_arr, sb->bit_mask);
 
-		ret = sb->ste_build_tag_func(value, sb, ste_arr);
+		ret = sb->ste_build_tag_func(value, sb, dr_ste_get_tag(ste_arr));
 		if (ret)
 			return ret;
 
@@ -837,8 +804,8 @@ int dr_ste_build_ste_arr(struct mlx5dv_dr_matcher *matcher,
 	return 0;
 }
 
-static int dr_ste_build_eth_l2_src_des_bit_mask(struct dr_match_param *value,
-						bool inner, uint8_t *bit_mask)
+static void dr_ste_build_eth_l2_src_des_bit_mask(struct dr_match_param *value,
+						 bool inner, uint8_t *bit_mask)
 {
 	struct dr_match_spec *mask = inner ? &value->inner : &value->outer;
 
@@ -866,13 +833,6 @@ static int dr_ste_build_eth_l2_src_des_bit_mask(struct dr_match_param *value,
 		DR_STE_SET(eth_l2_src_dst, bit_mask, first_vlan_qualifier, -1);
 		mask->svlan_tag = 0;
 	}
-
-	if (mask->cvlan_tag || mask->svlan_tag) {
-		errno = EINVAL;
-		return errno;
-	}
-
-	return 0;
 }
 
 static void dr_ste_copy_mask_misc(char *mask, struct dr_match_misc *spec)
@@ -1118,11 +1078,9 @@ void dr_ste_copy_param(uint8_t match_criteria,
 
 static int dr_ste_build_eth_l2_src_des_tag(struct dr_match_param *value,
 					   struct dr_ste_build *sb,
-					   uint8_t *hw_ste_p)
+					   uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_spec *spec = sb->inner ? &value->inner : &value->outer;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(eth_l2_src_dst, tag, dmac_47_16, spec, dmac_47_16);
 	DR_STE_SET_TAG(eth_l2_src_dst, tag, dmac_15_0, spec, dmac_15_0);
@@ -1163,23 +1121,17 @@ static int dr_ste_build_eth_l2_src_des_tag(struct dr_match_param *value,
 	return 0;
 }
 
-int dr_ste_build_eth_l2_src_des(struct dr_ste_build *sb,
-				struct dr_match_param *mask,
-				bool inner, bool rx)
+void dr_ste_build_eth_l2_src_dst(struct dr_ste_build *sb,
+				 struct dr_match_param *mask,
+				 bool inner, bool rx)
 {
-	int ret;
-
-	ret = dr_ste_build_eth_l2_src_des_bit_mask(mask, inner, sb->bit_mask);
-	if (ret)
-		return ret;
+	dr_ste_build_eth_l2_src_des_bit_mask(mask, inner, sb->bit_mask);
 
 	sb->rx = rx;
 	sb->inner = inner;
 	sb->lu_type = DR_STE_CALC_LU_TYPE(ETHL2_SRC_DST, rx, inner);
 	sb->byte_mask = dr_ste_conv_bit_to_byte_mask(sb->bit_mask);
 	sb->ste_build_tag_func = &dr_ste_build_eth_l2_src_des_tag;
-
-	return 0;
 }
 
 static void dr_ste_build_eth_l3_ipv6_dst_bit_mask(struct dr_match_param *value,
@@ -1195,11 +1147,9 @@ static void dr_ste_build_eth_l3_ipv6_dst_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_eth_l3_ipv6_dst_tag(struct dr_match_param *value,
 					    struct dr_ste_build *sb,
-					    uint8_t *hw_ste_p)
+					    uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_spec *spec = sb->inner ? &value->inner : &value->outer;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(eth_l3_ipv6_dst, tag, dst_ip_127_96, spec, dst_ip_127_96);
 	DR_STE_SET_TAG(eth_l3_ipv6_dst, tag, dst_ip_95_64, spec, dst_ip_95_64);
@@ -1235,11 +1185,9 @@ static void dr_ste_build_eth_l3_ipv6_src_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_eth_l3_ipv6_src_tag(struct dr_match_param *value,
 					    struct dr_ste_build *sb,
-					    uint8_t *hw_ste_p)
+					    uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_spec *spec = sb->inner ? &value->inner : &value->outer;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(eth_l3_ipv6_src, tag, src_ip_127_96, spec, src_ip_127_96);
 	DR_STE_SET_TAG(eth_l3_ipv6_src, tag, src_ip_95_64, spec, src_ip_95_64);
@@ -1287,11 +1235,9 @@ static void dr_ste_build_eth_l3_ipv4_5_tuple_bit_mask(struct dr_match_param *val
 
 static int dr_ste_build_eth_l3_ipv4_5_tuple_tag(struct dr_match_param *value,
 						struct dr_ste_build *sb,
-						uint8_t *hw_ste_p)
+						uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_spec *spec = sb->inner ? &value->inner : &value->outer;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(eth_l3_ipv4_5_tuple, tag, destination_address, spec, dst_ip_31_0);
 	DR_STE_SET_TAG(eth_l3_ipv4_5_tuple, tag, source_address, spec, src_ip_31_0);
@@ -1371,12 +1317,10 @@ dr_ste_build_eth_l2_src_or_dst_bit_mask(struct dr_match_param *value,
 }
 
 static int dr_ste_build_eth_l2_src_or_dst_tag(struct dr_match_param *value,
-					      bool inner, uint8_t *hw_ste_p)
+					      bool inner, uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_spec *spec = inner ? &value->inner : &value->outer;
 	struct dr_match_misc *misc_spec = &value->misc;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(eth_l2_src, tag, first_vlan_id, spec, first_vid);
 	DR_STE_SET_TAG(eth_l2_src, tag, first_cfi, spec, first_cfi);
@@ -1446,16 +1390,14 @@ static void dr_ste_build_eth_l2_src_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_eth_l2_src_tag(struct dr_match_param *value,
 				       struct dr_ste_build *sb,
-				       uint8_t *hw_ste_p)
+				       uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_spec *spec = sb->inner ? &value->inner : &value->outer;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(eth_l2_src, tag, smac_47_16, spec, smac_47_16);
 	DR_STE_SET_TAG(eth_l2_src, tag, smac_15_0, spec, smac_15_0);
 
-	return dr_ste_build_eth_l2_src_or_dst_tag(value, sb->inner, hw_ste_p);
+	return dr_ste_build_eth_l2_src_or_dst_tag(value, sb->inner, tag);
 }
 
 void dr_ste_build_eth_l2_src(struct dr_ste_build *sb,
@@ -1484,16 +1426,14 @@ static void dr_ste_build_eth_l2_dst_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_eth_l2_dst_tag(struct dr_match_param *value,
 				       struct dr_ste_build *sb,
-				       uint8_t *hw_ste_p)
+				       uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_spec *spec = sb->inner ? &value->inner : &value->outer;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(eth_l2_dst, tag, dmac_47_16, spec, dmac_47_16);
 	DR_STE_SET_TAG(eth_l2_dst, tag, dmac_15_0, spec, dmac_15_0);
 
-	return dr_ste_build_eth_l2_src_or_dst_tag(value, sb->inner, hw_ste_p);
+	return dr_ste_build_eth_l2_src_or_dst_tag(value, sb->inner, tag);
 }
 
 void dr_ste_build_eth_l2_dst(struct dr_ste_build *sb,
@@ -1538,12 +1478,10 @@ static void dr_ste_build_eth_l2_tnl_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_eth_l2_tnl_tag(struct dr_match_param *value,
 				       struct dr_ste_build *sb,
-				       uint8_t *hw_ste_p)
+				       uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_spec *spec = sb->inner ? &value->inner : &value->outer;
 	struct dr_match_misc *misc = &value->misc;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(eth_l2_tnl, tag, dmac_47_16, spec, dmac_47_16);
 	DR_STE_SET_TAG(eth_l2_tnl, tag, dmac_15_0, spec, dmac_15_0);
@@ -1605,11 +1543,9 @@ static void dr_ste_build_eth_l3_ipv4_misc_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_eth_l3_ipv4_misc_tag(struct dr_match_param *value,
 					     struct dr_ste_build *sb,
-					     uint8_t *hw_ste_p)
+					     uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_spec *spec = sb->inner ? &value->inner : &value->outer;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(eth_l3_ipv4_misc, tag, time_to_live, spec, ip_ttl_hoplimit);
 
@@ -1633,6 +1569,7 @@ static void dr_ste_build_ipv6_l3_l4_bit_mask(struct dr_match_param *value,
 					     bool inner, uint8_t *bit_mask)
 {
 	struct dr_match_spec *mask = inner ? &value->inner : &value->outer;
+	struct dr_match_misc *misc = &value->misc;
 
 	DR_STE_SET_MASK_V(eth_l4, bit_mask, dst_port, mask, tcp_dport);
 	DR_STE_SET_MASK_V(eth_l4, bit_mask, src_port, mask, tcp_sport);
@@ -1643,6 +1580,13 @@ static void dr_ste_build_ipv6_l3_l4_bit_mask(struct dr_match_param *value,
 	DR_STE_SET_MASK_V(eth_l4, bit_mask, dscp, mask, ip_dscp);
 	DR_STE_SET_MASK_V(eth_l4, bit_mask, ecn, mask, ip_ecn);
 	DR_STE_SET_MASK_V(eth_l4, bit_mask, ipv6_hop_limit, mask, ip_ttl_hoplimit);
+	if (inner) {
+		DR_STE_SET_MASK_V(eth_l4, bit_mask, flow_label,
+				  misc, inner_ipv6_flow_label);
+	} else {
+		DR_STE_SET_MASK_V(eth_l4, bit_mask, flow_label,
+				  misc, outer_ipv6_flow_label);
+	}
 
 	if (mask->tcp_flags) {
 		DR_STE_SET_TCP_FLAGS(eth_l4, bit_mask, mask);
@@ -1652,11 +1596,10 @@ static void dr_ste_build_ipv6_l3_l4_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_ipv6_l3_l4_tag(struct dr_match_param *value,
 				       struct dr_ste_build *sb,
-				       uint8_t *hw_ste_p)
+				       uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_spec *spec = sb->inner ? &value->inner : &value->outer;
-	uint8_t *tag = hw_ste->tag;
+	struct dr_match_misc *misc = &value->misc;
 
 	DR_STE_SET_TAG(eth_l4, tag, dst_port, spec, tcp_dport);
 	DR_STE_SET_TAG(eth_l4, tag, src_port, spec, tcp_sport);
@@ -1667,6 +1610,13 @@ static int dr_ste_build_ipv6_l3_l4_tag(struct dr_match_param *value,
 	DR_STE_SET_TAG(eth_l4, tag, dscp, spec, ip_dscp);
 	DR_STE_SET_TAG(eth_l4, tag, ecn, spec, ip_ecn);
 	DR_STE_SET_TAG(eth_l4, tag, ipv6_hop_limit, spec, ip_ttl_hoplimit);
+	if (sb->inner) {
+		DR_STE_SET_TAG(eth_l4, tag, flow_label,
+			       misc, inner_ipv6_flow_label);
+	} else {
+		DR_STE_SET_TAG(eth_l4, tag, flow_label,
+			       misc, outer_ipv6_flow_label);
+	}
 
 	if (spec->tcp_flags) {
 		DR_STE_SET_TCP_FLAGS(eth_l4, tag, spec);
@@ -1676,9 +1626,9 @@ static int dr_ste_build_ipv6_l3_l4_tag(struct dr_match_param *value,
 	return 0;
 }
 
-void dr_ste_build_ipv6_l3_l4(struct dr_ste_build *sb,
-			     struct dr_match_param *mask,
-			     bool inner, bool rx)
+void dr_ste_build_eth_ipv6_l3_l4(struct dr_ste_build *sb,
+				 struct dr_match_param *mask,
+				 bool inner, bool rx)
 {
 	dr_ste_build_ipv6_l3_l4_bit_mask(mask, inner, sb->bit_mask);
 
@@ -1691,7 +1641,7 @@ void dr_ste_build_ipv6_l3_l4(struct dr_ste_build *sb,
 
 static int dr_ste_build_empty_always_hit_tag(struct dr_match_param *value,
 					     struct dr_ste_build *sb,
-					     uint8_t *hw_ste_p)
+					     uint8_t *tag)
 {
 	return 0;
 }
@@ -1717,11 +1667,9 @@ static void dr_ste_build_mpls_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_mpls_tag(struct dr_match_param *value,
 				 struct dr_ste_build *sb,
-				 uint8_t *hw_ste_p)
+				 uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_misc2 *misc2_mask = &value->misc2;
-	uint8_t *tag = hw_ste->tag;
 
 	if (sb->inner)
 		DR_STE_SET_MPLS_TAG(mpls, misc2_mask, inner, tag);
@@ -1759,11 +1707,9 @@ static void dr_ste_build_gre_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_gre_tag(struct dr_match_param *value,
 				struct dr_ste_build *sb,
-				uint8_t *hw_ste_p)
+				uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct  dr_match_misc *misc = &value->misc;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(gre, tag, gre_protocol, misc, gre_protocol);
 
@@ -1778,8 +1724,8 @@ static int dr_ste_build_gre_tag(struct dr_match_param *value,
 	return 0;
 }
 
-void dr_ste_build_gre(struct dr_ste_build *sb, struct dr_match_param *mask,
-		      bool inner, bool rx)
+void dr_ste_build_tnl_gre(struct dr_ste_build *sb, struct dr_match_param *mask,
+			  bool inner, bool rx)
 {
 	dr_ste_build_gre_bit_mask(mask, inner, sb->bit_mask);
 
@@ -1824,11 +1770,9 @@ static void dr_ste_build_flex_parser_0_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_flex_parser_0_tag(struct dr_match_param *value,
 					  struct dr_ste_build *sb,
-					  uint8_t *hw_ste_p)
+					  uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_misc2 *misc_2_mask = &value->misc2;
-	uint8_t *tag = hw_ste->tag;
 
 	if (DR_STE_IS_OUTER_MPLS_OVER_GRE_SET(misc_2_mask)) {
 		DR_STE_SET_TAG(flex_parser_0, tag, parser_3_label,
@@ -1858,9 +1802,9 @@ static int dr_ste_build_flex_parser_0_tag(struct dr_match_param *value,
 	return 0;
 }
 
-void dr_ste_build_flex_parser_0(struct dr_ste_build *sb,
-				struct dr_match_param *mask,
-				bool inner, bool rx)
+void dr_ste_build_tnl_mpls(struct dr_ste_build *sb,
+			   struct dr_match_param *mask,
+			   bool inner, bool rx)
 {
 	dr_ste_build_flex_parser_0_bit_mask(mask, inner, sb->bit_mask);
 
@@ -1880,7 +1824,7 @@ static int dr_ste_build_flex_parser_1_bit_mask(struct dr_match_param *mask,
 					       uint8_t *bit_mask)
 {
 	struct dr_match_misc3 *misc_3_mask = &mask->misc3;
-	bool is_ipv4_mask = DR_MASK_IS_FLEX_PARSER_ICMPV4_SET(misc_3_mask);
+	bool is_ipv4_mask = DR_MASK_IS_ICMPV4_SET(misc_3_mask);
 	uint32_t icmp_header_data_mask;
 	uint32_t icmp_type_mask;
 	uint32_t icmp_code_mask;
@@ -1948,12 +1892,10 @@ static int dr_ste_build_flex_parser_1_bit_mask(struct dr_match_param *mask,
 
 static int dr_ste_build_flex_parser_1_tag(struct dr_match_param *value,
 					  struct dr_ste_build *sb,
-					  uint8_t *hw_ste_p)
+					  uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_misc3 *misc_3 = &value->misc3;
-	bool is_ipv4 = DR_MASK_IS_FLEX_PARSER_ICMPV4_SET(misc_3);
-	uint8_t *tag = hw_ste->tag;
+	bool is_ipv4 = DR_MASK_IS_ICMPV4_SET(misc_3);
 	uint32_t icmp_header_data;
 	uint32_t icmp_type;
 	uint32_t icmp_code;
@@ -2020,10 +1962,10 @@ static int dr_ste_build_flex_parser_1_tag(struct dr_match_param *value,
 	return 0;
 }
 
-int dr_ste_build_flex_parser_1(struct dr_ste_build *sb,
-			       struct dr_match_param *mask,
-			       struct dr_devx_caps *caps,
-			       bool inner, bool rx)
+int dr_ste_build_icmp(struct dr_ste_build *sb,
+		      struct dr_match_param *mask,
+		      struct dr_devx_caps *caps,
+		      bool inner, bool rx)
 {
 	int ret;
 
@@ -2053,11 +1995,9 @@ static void dr_ste_build_general_purpose_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_general_purpose_tag(struct dr_match_param *value,
 					    struct dr_ste_build *sb,
-					    uint8_t *hw_ste_p)
+					    uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_misc2 *misc_2_mask = &value->misc2;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(general_purpose, tag, general_purpose_lookup_field,
 		       misc_2_mask, metadata_reg_a);
@@ -2098,11 +2038,9 @@ static void dr_ste_build_eth_l4_misc_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_eth_l4_misc_tag(struct dr_match_param *value,
 					struct dr_ste_build *sb,
-					uint8_t *hw_ste_p)
+					uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_misc3 *misc3 = &value->misc3;
-	uint8_t *tag = hw_ste->tag;
 
 	if (sb->inner) {
 		DR_STE_SET_TAG(eth_l4_misc, tag, seq_num, misc3, inner_tcp_seq_num);
@@ -2148,11 +2086,9 @@ dr_ste_build_flex_parser_tnl_vxlan_gpe_bit_mask(struct dr_match_param *value,
 static int
 dr_ste_build_flex_parser_tnl_vxlan_gpe_tag(struct dr_match_param *value,
 					   struct dr_ste_build *sb,
-					   uint8_t *hw_ste_p)
+					   uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_misc3 *misc3 = &value->misc3;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(flex_parser_tnl_vxlan_gpe, tag,
 		       outer_vxlan_gpe_flags, misc3,
@@ -2167,9 +2103,9 @@ dr_ste_build_flex_parser_tnl_vxlan_gpe_tag(struct dr_match_param *value,
 	return 0;
 }
 
-void dr_ste_build_flex_parser_tnl_vxlan_gpe(struct dr_ste_build *sb,
-					    struct dr_match_param *mask,
-					    bool inner, bool rx)
+void dr_ste_build_tnl_vxlan_gpe(struct dr_ste_build *sb,
+				struct dr_match_param *mask,
+				bool inner, bool rx)
 {
 	dr_ste_build_flex_parser_tnl_vxlan_gpe_bit_mask(mask, inner,
 							sb->bit_mask);
@@ -2203,11 +2139,9 @@ dr_ste_build_flex_parser_tnl_geneve_bit_mask(struct dr_match_param *value,
 static int
 dr_ste_build_flex_parser_tnl_geneve_tag(struct dr_match_param *value,
 					struct dr_ste_build *sb,
-					uint8_t *hw_ste_p)
+					uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_misc *misc = &value->misc;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(flex_parser_tnl_geneve, tag,
 		       geneve_protocol_type, misc, geneve_protocol_type);
@@ -2221,9 +2155,9 @@ dr_ste_build_flex_parser_tnl_geneve_tag(struct dr_match_param *value,
 	return 0;
 }
 
-void dr_ste_build_flex_parser_tnl_geneve(struct dr_ste_build *sb,
-					 struct dr_match_param *mask,
-					 bool inner, bool rx)
+void dr_ste_build_tnl_geneve(struct dr_ste_build *sb,
+			     struct dr_match_param *mask,
+			     bool inner, bool rx)
 {
 	dr_ste_build_flex_parser_tnl_geneve_bit_mask(mask, sb->bit_mask);
 	sb->rx = rx;
@@ -2253,11 +2187,9 @@ dr_ste_build_flex_parser_tnl_gtpu_bit_mask(struct dr_match_param *value,
 static int
 dr_ste_build_flex_parser_tnl_gtpu_tag(struct dr_match_param *value,
 				      struct dr_ste_build *sb,
-				      uint8_t *hw_ste_p)
+				      uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_misc3 *misc3 = &value->misc3;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(flex_parser_tnl_gtpu, tag,
 		       gtpu_flags, misc3,
@@ -2272,9 +2204,9 @@ dr_ste_build_flex_parser_tnl_gtpu_tag(struct dr_match_param *value,
 	return 0;
 }
 
-void dr_ste_build_flex_parser_tnl_gtpu(struct dr_ste_build *sb,
-				       struct dr_match_param *mask,
-				       bool inner, bool rx)
+void dr_ste_build_tnl_gtpu(struct dr_ste_build *sb,
+			   struct dr_match_param *mask,
+			   bool inner, bool rx)
 {
 	dr_ste_build_flex_parser_tnl_gtpu_bit_mask(mask, sb->bit_mask);
 	sb->rx = rx;
@@ -2301,11 +2233,9 @@ static void dr_ste_build_register_0_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_register_0_tag(struct dr_match_param *value,
 				       struct dr_ste_build *sb,
-				       uint8_t *hw_ste_p)
+				       uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_misc2 *misc2 = &value->misc2;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(register_0, tag, register_0_h, misc2, metadata_reg_c_0);
 	DR_STE_SET_TAG(register_0, tag, register_0_l, misc2, metadata_reg_c_1);
@@ -2345,11 +2275,9 @@ static void dr_ste_build_register_1_bit_mask(struct dr_match_param *value,
 
 static int dr_ste_build_register_1_tag(struct dr_match_param *value,
 				       struct dr_ste_build *sb,
-				       uint8_t *hw_ste_p)
+				       uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_misc2 *misc2 = &value->misc2;
-	uint8_t *tag = hw_ste->tag;
 
 	DR_STE_SET_TAG(register_1, tag, register_2_h, misc2, metadata_reg_c_4);
 	DR_STE_SET_TAG(register_1, tag, register_2_l, misc2, metadata_reg_c_5);
@@ -2372,54 +2300,48 @@ void dr_ste_build_register_1(struct dr_ste_build *sb,
 	sb->ste_build_tag_func = &dr_ste_build_register_1_tag;
 }
 
-static int dr_ste_build_src_gvmi_qpn_bit_mask(struct dr_match_param *value,
-					      uint8_t *bit_mask)
+static void dr_ste_build_src_gvmi_qpn_bit_mask(struct dr_match_param *value,
+					       uint8_t *bit_mask)
 {
 	struct dr_match_misc *misc_mask = &value->misc;
 
-	if (misc_mask->source_port && misc_mask->source_port != 0xffff) {
-		errno = EINVAL;
-		return errno;
-	}
 	DR_STE_SET_MASK(src_gvmi_qp, bit_mask, source_gvmi, misc_mask, source_port);
 	DR_STE_SET_MASK(src_gvmi_qp, bit_mask, source_qp, misc_mask, source_sqn);
-
-	return 0;
 }
 
 static int dr_ste_build_src_gvmi_qpn_tag(struct dr_match_param *value,
 					 struct dr_ste_build *sb,
-					 uint8_t *hw_ste_p)
+					 uint8_t *tag)
 {
-	struct dr_hw_ste_format *hw_ste = (struct dr_hw_ste_format *)hw_ste_p;
 	struct dr_match_misc *misc = &value->misc;
 	struct dr_devx_vport_cap *vport_cap;
-	uint8_t *tag = hw_ste->tag;
+	uint8_t *bit_mask = sb->bit_mask;
+	bool source_gvmi_set;
 
 	DR_STE_SET_TAG(src_gvmi_qp, tag, source_qp, misc, source_sqn);
 
-	vport_cap = dr_get_vport_cap(sb->caps, misc->source_port);
-	if (!vport_cap)
-		return errno;
+	source_gvmi_set = DR_STE_GET(src_gvmi_qp, bit_mask, source_gvmi);
+	if (source_gvmi_set) {
+		vport_cap = dr_get_vport_cap(sb->caps, misc->source_port);
+		if (!vport_cap)
+			return errno;
 
-	if (vport_cap->gvmi)
-		DR_STE_SET(src_gvmi_qp, tag, source_gvmi, vport_cap->gvmi);
+		if (vport_cap->gvmi)
+			DR_STE_SET(src_gvmi_qp, tag, source_gvmi,
+				   vport_cap->gvmi);
 
-	misc->source_port = 0;
+		misc->source_port = 0;
+	}
 
 	return 0;
 }
 
-int dr_ste_build_src_gvmi_qpn(struct dr_ste_build *sb,
-			      struct dr_match_param *mask,
-			      struct dr_devx_caps *caps,
-			      bool inner, bool rx)
+void dr_ste_build_src_gvmi_qpn(struct dr_ste_build *sb,
+			       struct dr_match_param *mask,
+			       struct dr_devx_caps *caps,
+			       bool inner, bool rx)
 {
-	int ret;
-
-	ret = dr_ste_build_src_gvmi_qpn_bit_mask(mask, sb->bit_mask);
-	if (ret)
-		return ret;
+	dr_ste_build_src_gvmi_qpn_bit_mask(mask, sb->bit_mask);
 
 	sb->rx = rx;
 	sb->caps = caps;
@@ -2427,6 +2349,4 @@ int dr_ste_build_src_gvmi_qpn(struct dr_ste_build *sb,
 	sb->lu_type = DR_STE_LU_TYPE_SRC_GVMI_AND_QP;
 	sb->byte_mask = dr_ste_conv_bit_to_byte_mask(sb->bit_mask);
 	sb->ste_build_tag_func = &dr_ste_build_src_gvmi_qpn_tag;
-
-	return 0;
 }
