@@ -51,6 +51,10 @@
 
 #define PFX		"mlx5: "
 
+#ifndef PCI_VENDOR_ID_MELLANOX
+#define PCI_VENDOR_ID_MELLANOX 0x15b3
+#endif
+
 typedef _Atomic(uint32_t) atomic_uint32_t;
 
 enum {
@@ -232,6 +236,13 @@ struct mlx5_uar_info {
 
 enum mlx5_ctx_flags {
 	MLX5_CTX_FLAGS_FATAL_STATE = 1 << 0,
+	MLX5_CTX_FLAGS_NO_KERN_DYN_UAR = 1 << 1,
+	MLX5_CTX_FLAGS_ECE_SUPPORTED = 1 << 2,
+};
+
+struct mlx5_lag_caps {
+	uint8_t num_lag_ports;
+	uint8_t lag_tx_port_affinity;
 };
 
 struct mlx5_context {
@@ -302,16 +313,27 @@ struct mlx5_context {
 	struct mlx5dv_striding_rq_caps	striding_rq_caps;
 	uint32_t			tunnel_offloads_caps;
 	struct mlx5_packet_pacing_caps	packet_pacing_caps;
+	struct mlx5_lag_caps		lag_caps;
 	pthread_mutex_t			dyn_bfregs_mutex; /* protects the dynamic bfregs allocation */
 	uint32_t			num_dyn_bfregs;
-	uint32_t			*count_dyn_bfregs;
-	uint32_t			start_dyn_bfregs_index;
+	uint32_t			max_num_legacy_dyn_uar_sys_page;
+	uint32_t			curr_legacy_dyn_sys_uar_page;
 	uint16_t			flow_action_flags;
 	uint64_t			max_dm_size;
 	uint32_t                        eth_min_inline_size;
 	uint32_t                        dump_fill_mkey;
 	__be32                          dump_fill_mkey_be;
 	uint32_t			flags;
+	struct list_head		dyn_uar_bf_list;
+	struct list_head		dyn_uar_nc_list;
+	struct list_head		dyn_uar_qp_shared_list;
+	struct list_head		dyn_uar_qp_dedicated_list;
+	uint16_t			qp_max_dedicated_uuars;
+	uint16_t			qp_alloc_dedicated_uuars;
+	uint16_t			qp_max_shared_uuars;
+	uint16_t			qp_alloc_shared_uuars;
+	struct mlx5_bf			*cq_uar;
+	void				*cq_uar_reg;
 };
 
 struct mlx5_bitmap {
@@ -380,8 +402,7 @@ enum {
 };
 
 struct mlx5_cq {
-	/* ibv_cq should always be subset of ibv_cq_ex */
-	struct ibv_cq_ex		ibv_cq;
+	struct verbs_cq			verbs_cq;
 	struct mlx5_buf			buf_a;
 	struct mlx5_buf			buf_b;
 	struct mlx5_buf		       *active_buf;
@@ -392,6 +413,7 @@ struct mlx5_cq {
 	uint32_t			cqn;
 	uint32_t			cons_index;
 	__be32			       *dbrec;
+	bool				custom_db;
 	int				arm_sn;
 	int				cqe_sz;
 	int				resize_cqe_sz;
@@ -406,6 +428,7 @@ struct mlx5_cq {
 	uint32_t			flags;
 	int			umr_opcode;
 	struct mlx5dv_clock_info	last_clock_info;
+	struct ibv_pd			*parent_domain;
 };
 
 struct mlx5_tag_entry {
@@ -503,6 +526,16 @@ struct mlx5_bf {
 	/* Index in the dynamic bfregs portion */
 	uint32_t			bfreg_dyn_index;
 	struct mlx5_devx_uar		devx_uar;
+	uint8_t				dyn_alloc_uar : 1;
+	uint8_t				mmaped_entry : 1;
+	uint8_t				nc_mode : 1;
+	uint8_t				qp_dedicated : 1;
+	uint8_t				qp_shared : 1;
+	uint32_t			count;
+	struct list_node		uar_entry;
+	uint32_t			uar_handle;
+	uint32_t			length;
+	uint32_t			page_id;
 };
 
 struct mlx5_dm {
@@ -515,7 +548,6 @@ struct mlx5_dm {
 
 struct mlx5_mr {
 	struct verbs_mr                 vmr;
-	struct mlx5_buf			buf;
 	uint32_t			alloc_flags;
 };
 
@@ -570,6 +602,19 @@ struct mlx5_qp {
 	uint32_t			rqn;
 	uint32_t			sqn;
 	uint64_t			tir_icm_addr;
+	/*
+	 * ECE configuration is done in create/modify QP stages,
+	 * so this value is cached version of the requested ECE prior
+	 * to its execution. This field will be cleared after successful
+	 * call to relevant "executor".
+	 */
+	uint32_t			set_ece;
+	/*
+	 * This field indicates returned ECE options from the device
+	 * as were received from the HW in previous stage. Every
+	 * write to the set_ece will clear this field.
+	 */
+	uint32_t			get_ece;
 };
 
 struct mlx5_ah {
@@ -622,6 +667,7 @@ enum mlx5_devx_obj_type {
 	MLX5_DEVX_FLOW_METER		= 3,
 	MLX5_DEVX_QP			= 4,
 	MLX5_DEVX_PKT_REFORMAT_CTX	= 5,
+	MLX5_DEVX_TIR			= 6,
 };
 
 struct mlx5dv_devx_obj {
@@ -629,6 +675,7 @@ struct mlx5dv_devx_obj {
 	uint32_t handle;
 	enum mlx5_devx_obj_type type;
 	uint32_t object_id;
+	uint64_t rx_icm_addr;
 };
 
 struct mlx5_var_obj {
@@ -637,6 +684,12 @@ struct mlx5_var_obj {
 	uint32_t handle;
 };
 
+struct mlx5_pp_obj {
+	struct mlx5dv_pp dv_pp;
+	struct ibv_context *context;
+	uint32_t handle;
+};
+
 struct mlx5_devx_umem {
 	struct mlx5dv_devx_umem dv_devx_umem;
 	struct ibv_context *context;
@@ -714,7 +767,7 @@ static inline struct mlx5_parent_domain *to_mparent_domain(struct ibv_pd *ibpd)
 
 static inline struct mlx5_cq *to_mcq(struct ibv_cq *ibcq)
 {
-	return container_of((struct ibv_cq_ex *)ibcq, struct mlx5_cq, ibv_cq);
+	return container_of(ibcq, struct mlx5_cq, verbs_cq.cq);
 }
 
 static inline struct mlx5_srq *to_msrq(struct ibv_srq *ibsrq)
@@ -972,6 +1025,7 @@ struct ibv_pd *mlx5_alloc_parent_domain(struct ibv_context *context,
 
 void *mlx5_mmap(struct mlx5_uar_info *uar, int index,
 		int cmd_fd, int page_size, int uar_type);
+off_t get_uar_mmap_offset(int idx, int page_size, int command);
 
 struct ibv_counters *mlx5_create_counters(struct ibv_context *context,
 					  struct ibv_counters_init_attr *init_attr);
@@ -988,9 +1042,21 @@ int mlx5_advise_mr(struct ibv_pd *pd,
 		   uint32_t flags,
 		   struct ibv_sge *sg_list,
 		   uint32_t num_sges);
+struct ibv_mr *mlx5_import_mr(struct ibv_pd *pd,
+			      uint32_t mr_handle);
+void mlx5_unimport_mr(struct ibv_mr *mr);
+struct ibv_pd *mlx5_import_pd(struct ibv_context *context,
+			      uint32_t pd_handle);
+void mlx5_unimport_pd(struct ibv_pd *pd);
 int mlx5_qp_fill_wr_pfns(struct mlx5_qp *mqp,
 			 const struct ibv_qp_init_attr_ex *attr,
 			 const struct mlx5dv_qp_init_attr *mlx5_attr);
+void clean_dyn_uars(struct ibv_context *context);
+struct mlx5_bf *mlx5_attach_dedicated_uar(struct ibv_context *context,
+					  uint32_t flags);
+
+int mlx5_set_ece(struct ibv_qp *qp, struct ibv_ece *ece);
+int mlx5_query_ece(struct ibv_qp *qp, struct ibv_ece *ece);
 
 static inline void *mlx5_find_uidx(struct mlx5_context *ctx, uint32_t uidx)
 {
