@@ -1,21 +1,22 @@
 # SPDX-License-Identifier: (GPL-2.0 OR Linux-OpenIB)
 # Copyright (c) 2019 Mellanox Technologies, Inc . All rights reserved. See COPYING file
 
+import subprocess
 import unittest
 import tempfile
 import random
 import errno
 import stat
+import json
 import os
 
-from pyverbs.pyverbs_error import PyverbsRDMAError, PyverbsUserError
 from pyverbs.qp import QPCap, QPInitAttrEx, QPInitAttr, QPAttr, QP
-from pyverbs.cmid import CMID, AddrInfo, CMEventChannel, ConnParam
+from pyverbs.pyverbs_error import PyverbsRDMAError
 from pyverbs.addr import AHAttr, GlobalRoute
 from pyverbs.xrcd import XRCD, XRCDInitAttr
 from pyverbs.srq import SRQ, SrqInitAttrEx
 from pyverbs.device import Context
-import pyverbs.cm_enums as ce
+from args_parser import parser
 import pyverbs.device as d
 import pyverbs.enums as e
 from pyverbs.pd import PD
@@ -30,22 +31,46 @@ MIN_RNR_TIMER =12
 RETRY_CNT = 7
 RNR_RETRY = 7
 TIMEOUT = 14
+# Devices that don't support RoCEv2 should be added here
+MLNX_VENDOR_ID = 0x02c9
+CX3_MLNX_PART_ID = 4099
+CX3Pro_MLNX_PART_ID = 4103
+# Dictionary: vendor_id -> array of part_ids of devices that lack RoCEv2 support
+ROCEV2_UNSUPPORTED_DEVS = {MLNX_VENDOR_ID: [CX3Pro_MLNX_PART_ID,
+                                            CX3_MLNX_PART_ID]}
+
+
+def has_roce_hw_bug(vendor_id, vendor_part_id):
+    return vendor_part_id in ROCEV2_UNSUPPORTED_DEVS.get(vendor_id, [])
 
 
 class PyverbsAPITestCase(unittest.TestCase):
+    def __init__(self, methodName='runTest'):
+        super().__init__(methodName)
+        # Hold the command line arguments
+        self.config = parser.get_config()
+
     def setUp(self):
         """
         Opens the devices and queries them
         """
-        lst = d.get_device_list()
         self.devices = []
-        if len(lst) == 0:
-            raise unittest.SkipTest('No IB devices found')
-        for dev in lst:
-            c = d.Context(name=dev.name.decode())
+
+        dev_name = self.config['dev']
+        if dev_name:
+            c = d.Context(name=dev_name)
             attr = c.query_device()
             attr_ex = c.query_device_ex()
             self.devices.append((c, attr, attr_ex))
+        else:
+            for dev in d.get_device_list():
+                c = d.Context(name=dev.name.decode())
+                attr = c.query_device()
+                attr_ex = c.query_device_ex()
+                self.devices.append((c, attr, attr_ex))
+
+        if len(self.devices) == 0:
+            raise unittest.SkipTest('No IB devices found')
 
     def tearDown(self):
         for tup in self.devices:
@@ -68,10 +93,28 @@ class RDMATestCase(unittest.TestCase):
     def __init__(self, methodName='runTest', dev_name=None, ib_port=None,
                  gid_index=None, pkey_index=None):
         super(RDMATestCase, self).__init__(methodName)
-        self.dev_name = dev_name
+        # Hold the command line arguments
+        self.config = parser.get_config()
+        dev = self.config['dev']
+        self.dev_name = dev_name if dev_name else dev
         self.ib_port = ib_port
         self.gid_index = gid_index
         self.pkey_index = pkey_index
+        self.ip_addr = None
+
+    def is_eth_and_has_roce_hw_bug(self):
+        """
+        Check if the link layer is Ethernet and the device lacks RoCEv2 support
+        with a known HW bug.
+        return: True if the link layer is Ethernet and device is not supported
+        """
+        ctx = d.Context(name=self.dev_name)
+        port_attrs = ctx.query_port(self.ib_port)
+        dev_attrs = ctx.query_device()
+        vendor_id = dev_attrs.vendor_id
+        vendor_pid = dev_attrs.vendor_part_id
+        return port_attrs.link_layer == e.IBV_LINK_LAYER_ETHERNET and \
+            has_roce_hw_bug(vendor_id, vendor_pid)
 
     @staticmethod
     def parametrize(testcase_klass, dev_name=None, ib_port=None, gid_index=None,
@@ -89,6 +132,22 @@ class RDMATestCase(unittest.TestCase):
                                          pkey_index=pkey_index))
         return suite
 
+    @staticmethod
+    def get_net_name(dev):
+        out = subprocess.check_output(['ls',
+                                       '/sys/class/infiniband/{}/device/net/'
+                                      .format(dev)])
+        return out.decode().split('\n')[0]
+
+    @staticmethod
+    def get_ip_address(ifname):
+        out = subprocess.check_output(['ip', '-j', 'addr', 'show', ifname])
+        loaded_json = json.loads(out.decode())
+        interface = loaded_json[0]['addr_info'][0]['local']
+        if 'fe80::' in interface:
+            interface = interface + '%' + ifname
+        return interface
+
     def setUp(self):
         """
         Verify that the test case has dev_name, ib_port, gid_index and pkey index.
@@ -124,102 +183,57 @@ class RDMATestCase(unittest.TestCase):
         if not self.args:
             raise unittest.SkipTest('No port is up, can\'t run traffic')
         # Choose one combination and use it
-        args = random.choice(self.args)
-        self.dev_name = args[0]
-        self.ib_port = args[1]
-        self.gid_index = args[2]
+        self._select_config()
+        self.dev_info = {'dev_name': self.dev_name, 'ib_port': self.ib_port,
+                         'gid_index': self.gid_index}
 
     def _add_gids_per_port(self, ctx, dev, port):
         # Don't add ports which are not active
-        if ctx.query_port(port).state != e.IBV_PORT_ACTIVE:
+        port_attrs = ctx.query_port(port)
+        if port_attrs.state != e.IBV_PORT_ACTIVE:
             return
-
-        for idx in range(ctx.query_port(port).gid_tbl_len):
+        dev_attrs = ctx.query_device()
+        vendor_id = dev_attrs.vendor_id
+        vendor_pid = dev_attrs.vendor_part_id
+        for idx in range(port_attrs.gid_tbl_len):
             gid = ctx.query_gid(port, idx)
             # Avoid adding ZERO GIDs
-            if gid.gid[-19:] != self.ZERO_GID:
-                self.args.append([dev, port, idx])
+            if gid.gid[-19:] == self.ZERO_GID:
+                continue
+            # Avoid RoCEv2 GIDs on unsupported devices
+            if port_attrs.link_layer == e.IBV_LINK_LAYER_ETHERNET and \
+                    ctx.query_gid_type(port, idx) == e.IBV_GID_TYPE_ROCE_V2 and \
+                    has_roce_hw_bug(vendor_id, vendor_pid):
+                continue
+            if not os.path.exists('/sys/class/infiniband/{}/device/net/'.format(dev)):
+                self.args.append([dev, port, idx, None])
+                continue
+            net_name = self.get_net_name(dev)
+            try:
+                ip_addr = self.get_ip_address(net_name)
+            except (KeyError, IndexError):
+                self.args.append([dev, port, idx, None])
+            else:
+                self.args.append([dev, port, idx, ip_addr])
 
     def _add_gids_per_device(self, ctx, dev):
         port_count = ctx.query_device().phys_port_cnt
         for port in range(port_count):
             self._add_gids_per_port(ctx, dev, port+1)
 
-
-class CMResources:
-    """
-    CMResources class is a base aggregator object which contains basic
-    resources for RDMA CM communication.
-    """
-    def __init__(self, **kwargs):
-        """
-        :param kwargs: Arguments:
-            * *src* (str)
-               Local address to bind to (for passive side)
-            * *dst* (str)
-               Destination address to connect (for active side)
-            * *port* (str)
-                Port number of the address
-            * *is_async* (bool)
-                A flag which indicates if its asynchronous RDMACM
-        """
-        src = kwargs.get('src')
-        dst = kwargs.get('dst')
-        self.is_server = True if dst is None else False
-        self.qp_init_attr = None
-        self.is_async = kwargs.get('is_async', False)
-        self.connected = False
-        # When passive side (server) listens to incoming connection requests,
-        # for each new request it creates a new cmid which is used to establish
-        # the connection with the remote side
-        self.child_id = None
-        self.msg_size = 1024
-        self.num_msgs = 100
-        self.channel = None
-        self.port = kwargs.get('port') if kwargs.get('port') else '7471'
-        self.mr = None
-        if self.is_server:
-            self.ai = AddrInfo(src, None, self.port, ce.RDMA_PS_TCP,
-                               ce.RAI_PASSIVE)
+    def _select_config(self):
+        args_with_inet_ip = []
+        for arg in self.args:
+            if arg[3]:
+                args_with_inet_ip.append(arg)
+        if args_with_inet_ip:
+            args = random.choice(args_with_inet_ip)
         else:
-            self.ai = AddrInfo(src, dst, self.port, ce.RDMA_PS_TCP)
-        if self.is_async:
-            self.create_event_channel()
-            self.cmid = CMID(creator=self.channel)
-        else:
-            self.cmid = CMID(creator=self.ai,
-                             qp_init_attr=self.create_qp_init_attr())
-
-    def create_mr(self):
-        if self.is_server:
-            self.mr = self.child_id.reg_msgs(self.msg_size)
-        else:
-            self.mr = self.cmid.reg_msgs(self.msg_size)
-
-    def create_event_channel(self):
-        self.channel = CMEventChannel()
-
-    @staticmethod
-    def create_qp_init_attr():
-        return QPInitAttr(qp_type=e.IBV_QPT_RC, cap=QPCap(max_recv_wr=1))
-
-    @staticmethod
-    def create_conn_param():
-        return ConnParam()
-
-    def create_child_id(self, cm_event=None):
-        if not self.is_server:
-            raise PyverbsUserError('create_child_id can be used only in passive side')
-        if self.is_async:
-            self.child_id = CMID(creator=cm_event, listen_id=self.cmid)
-        else:
-            self.child_id = self.cmid.get_request()
-
-    def create_qp(self):
-        if self.is_server:
-            self.child_id.create_qp(self.create_qp_init_attr())
-        else:
-            self.cmid.create_qp(self.create_qp_init_attr())
+            args = random.choice(self.args)
+        self.dev_name = args[0]
+        self.ib_port = args[1]
+        self.gid_index = args[2]
+        self.ip_addr = args[3]
 
 
 class BaseResources(object):
@@ -235,10 +249,17 @@ class BaseResources(object):
         :param ib_port: IB port of the device to use (default: 1)
         :param gid_index: Which GID index to use (default: 0)
         """
-        self.ctx = Context(name=dev_name)
+        self.dev_name = dev_name
         self.gid_index = gid_index
-        self.pd = PD(self.ctx)
         self.ib_port = ib_port
+        self.create_context()
+        self.create_pd()
+
+    def create_context(self):
+        self.ctx = Context(name=self.dev_name)
+
+    def create_pd(self):
+        self.pd = PD(self.ctx)
 
 
 class TrafficResources(BaseResources):
@@ -262,6 +283,7 @@ class TrafficResources(BaseResources):
         self.num_msgs = 1000
         self.port_attr = None
         self.mr = None
+        self.use_mr_prefetch = None
         self.cq = None
         self.qp = None
         self.rqpn = 0
